import type { Readable, StartStopNotifier, Writable } from "svelte/store";
export declare const Refuse: unique symbol;
export declare type Refuse = typeof Refuse;
export declare type Key = string | number | symbol;
export declare type Accessor<P, C> = {
    readChild: (parent: P) => C | Refuse;
    writeChild: (parent: P, newChild: C) => void;
};
export declare function objectAccessor<P, K extends keyof P>(key: K): Accessor<P, P[K]>;
export declare function mapAccessor<K extends Key, V>(key: K): Accessor<Map<K, V>, V>;
export declare type ReadableTreeCore<P> = {
    zoom<C>(accessor: Accessor<P, C>): ReadableTree<C>;
    zoomIn<K extends keyof P>(k: K): ReadableTree<P[K]>;
    choose<P_ extends P>(chooseParent: (parent: P) => P_ | Refuse): ReadableTree<P_>;
};
export declare type WritableTreeCore<P> = ReadableTreeCore<P> & {
    zoomWritable<C>(accessor: Accessor<P, C>): WritableTree<C>;
    zoomInWritable<K extends keyof P>(k: K): WritableTree<P[K]>;
    chooseWritable<P_ extends P>(chooseParent: (parent: P) => P_ | Refuse): WritableTree<P_>;
};
export declare type ReadableTree<P> = Readable<P> & ReadableTreeCore<P>;
export declare type WritableTree<P> = Writable<P> & WritableTreeCore<P>;
export declare function readableTree<P>(value: P, start?: StartStopNotifier<P>): ReadableTree<P>;
export declare function writableTree<P>(value: P, start?: StartStopNotifier<P>): WritableTree<P>;
